<?php

if (!defined('ABSPATH')) {
    exit; // Exit if accessed directly
}

/**
 * Class AFRSM_Shipping_Method.
 *
 * WooCommerce Advanced flat rate shipping method class.
 */
if (class_exists('AFRSM_Shipping_Method'))
    return; // Stop if the class already exists

class AFRSM_Shipping_Method extends WC_Shipping_Method
{

    /**
     * Constructor
     *
     * @since 3.0.0
     */
    public function __construct()
    {

        $post_title = isset($_REQUEST['id']) ? get_the_title($_REQUEST['id']) : '';

        $shipping_method_id = isset($_REQUEST['id']) && !empty($_REQUEST['id']) ? $_REQUEST['id'] : 'advanced_flat_rate_shipping';
        $shipping_method_title = !empty($post_title) ? $post_title : 'Advanced Flat Rate Shipping';

        $this->id = $shipping_method_id;
        $this->title = __('Advanced Flat Rate Shipping', AFRSM_PRO_TEXT_DOMAIN);
        $this->method_title = __($shipping_method_title, AFRSM_PRO_TEXT_DOMAIN);

        $this->init();

        // Save settings
        add_action('woocommerce_update_options_shipping_' . $this->id, array($this, 'process_admin_options'));
    }

    /**
     * Init
     *
     * @since 3.0.0
     */
    function init()
    {
        $this->init_form_fields();
        $this->init_settings();
    }

    /**
     * Init form fields.
     *
     * @since 3.0.0
     */
    public function init_form_fields()
    {
        $this->form_fields = array(
            'advanced_flat_rate_shipping_table' => array(
                'type' => 'advanced_flat_rate_shipping_table',
            ),
        );
    }

    /**
     * General advanced flat rate settings tab table.
     *
     * @since 3.0.0
     */
    public function generate_advanced_flat_rate_shipping_table_html()
    {
        ob_start();
        require plugin_dir_path(__FILE__) . 'afrsm-pro-list-page.php';
        return ob_get_clean();
    }

    /**
     * Calculate shipping.
     *
     * @since 3.0.0
     *
     * @param array $package List containing all products for this method.
     */
    public function calculate_shipping($package = array())
    {
        global $sitepress, $woocommerce, $woocommerce_wpml;

        if (!empty($sitepress)) {
            $default_lang = $sitepress->get_default_language();
        }
        $matched_shipping_methods = $this->afrsm_match_methods($package);

        if ($matched_shipping_methods == false || !is_array($matched_shipping_methods)) {
            return;
        }

        $cart_array = $woocommerce->cart->get_cart();

        /* Strat ordering issue and highest, smallest, forceall shipping issue code */
        $costs_array = array();
        $get_what_to_do_method = get_option('what_to_do_method');
        $get_what_to_do_method = !empty($get_what_to_do_method) ? $get_what_to_do_method : 'allow_customer';
        $combine_default_shipping_with_forceall = get_option('combine_default_shipping_with_forceall');

        if ($get_what_to_do_method == 'allow_customer') {
            if (!empty($matched_shipping_methods)) {
                $getSortOrder = get_option('sm_sortable_order');
                $sort_order = array();
                if (!empty($getSortOrder)) {
                    foreach ($getSortOrder as $getSortOrder_id) {
                        if (in_array($getSortOrder_id, $matched_shipping_methods)) {
                            $sort_order[] = $getSortOrder_id;
                        }
                    }
                    unset($matched_shipping_methods);
                    $matched_shipping_methods = $sort_order;
                }
            }
        }

        if ($get_what_to_do_method == 'apply_smallest') {
            if (!empty($matched_shipping_methods)) {
                foreach ($matched_shipping_methods as $shipping_method_id) {
                    $shipping_title = get_the_title($shipping_method_id);

                    $shipping_rate = array(
                        'id' => $shipping_method_id,
                        'label' => $shipping_title,
                        'cost' => 0,
                    );

                    $costs = get_post_meta($shipping_method_id, 'sm_product_cost', true);
                    $getFeesPerQtyFlag = get_post_meta($shipping_method_id, 'sm_fee_chk_qty_price', true);
                    $getFeesPerQty = get_post_meta($shipping_method_id, 'sm_fee_per_qty', true);
                    $extraProductCostOriginal = get_post_meta($shipping_method_id, 'sm_extra_product_cost', true);

                    $cart_based_qty = '0';
                    $products_based_qty = '0';
                    if ($getFeesPerQtyFlag == 'on') {
                        if ($getFeesPerQty == 'qty_cart_based') {
                            /*Start : Cart Based quantity for Apply Highest*/
                            if (!empty($cart_array)) {
                                foreach ($cart_array as $woo_cart_item_key => $value) {
                                    $cart_based_qty += $value['quantity'];
                                }
                            }
                            /*End : Cart Based quantity for Apply Highest*/
                        }
                        if ($getFeesPerQty == 'qty_product_based') {
                            /*Start : Product Based quantity for Apply Highest*/
                            $productFeesArray = get_post_meta($shipping_method_id, 'sm_metabox', true);
                            /* @var $productFeesArray type */
                            $check_prd_qty = array();
                            if (!empty($productFeesArray)) {
                                foreach ($productFeesArray as $key => $condition) {
                                    if (array_search('product', $condition)) {
                                        $site_product_id = '';
                                        $cart_final_products_array = array();
                                        /* Product Condition Start */
                                        if ($condition['product_fees_conditions_is'] == 'is_equal_to') {
                                            if (!empty($condition['product_fees_conditions_values'])) {
                                                foreach ($condition['product_fees_conditions_values'] as $product_id) {
                                                    foreach ($cart_array as $key => $value) {
                                                        if (!empty($check_prd_qty)) {
                                                            if (!array_key_exists($value['product_id'], $check_prd_qty)) {
                                                                if (!empty($sitepress)) {
                                                                    $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                                } else {
                                                                    $site_product_id = $value['product_id'];
                                                                }

                                                                if ($product_id == $site_product_id) {
                                                                    $cart_final_products_array[] = $value;
                                                                }
                                                            }
                                                        } else {
                                                            if (!empty($sitepress)) {
                                                                $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                            } else {
                                                                $site_product_id = $value['product_id'];
                                                            }

                                                            if ($product_id == $site_product_id) {
                                                                $cart_final_products_array[] = $value;
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        } elseif ($condition['product_fees_conditions_is'] == 'not_in') {
                                            if (!empty($condition['product_fees_conditions_values'])) {
                                                foreach ($condition['product_fees_conditions_values'] as $product_id) {
                                                    foreach ($cart_array as $key => $value) {
                                                        if (!empty($check_prd_qty)) {
                                                            if (!array_key_exists($value['product_id'], $check_prd_qty)) {
                                                                if (!empty($sitepress)) {
                                                                    $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                                } else {
                                                                    $site_product_id = $value['product_id'];
                                                                }

                                                                if ($product_id != $site_product_id) {
                                                                    $cart_final_products_array[$product_id] = $value;
                                                                }
                                                            }
                                                        } else {
                                                            if (!empty($sitepress)) {
                                                                $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                            } else {
                                                                $site_product_id = $value['product_id'];
                                                            }

                                                            if ($product_id != $site_product_id) {
                                                                $cart_final_products_array[$product_id] = $value;
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }

                                        if (!empty($cart_final_products_array)) {
                                            foreach ($cart_final_products_array as $cart_prd_id => $cart_item) {
	                                            $product_cart_array = new WC_Product($cart_item['product_id']);
	                                            if (!($product_cart_array->is_virtual('yes'))) {
		                                            $products_based_qty += $cart_item['quantity'];
	                                            }
                                            }
                                        }
                                    }
                                }
                            }
                            /*End : Product Based quantity for Apply Highest*/
                        }
                    }

                    // Calculate the costs
                    if (isset($woocommerce_wpml) && !empty($woocommerce_wpml->multi_currency)) {
                        $extraProductCost = $woocommerce_wpml->multi_currency->prices->convert_price_amount($extraProductCostOriginal);
                    } else {
                        $extraProductCost = $extraProductCostOriginal;
                    }

                    if ($getFeesPerQtyFlag == 'on') {
                        if ($getFeesPerQty == 'qty_cart_based') {
                            $costs = $costs + (($cart_based_qty - 1) * $extraProductCost);
                        } elseif ($getFeesPerQty == 'qty_product_based') {
                            $costs = $costs + (($products_based_qty - 1) * $extraProductCost);
                        }
                    }
                    $costs_array[$shipping_method_id] = $costs;
                }
                $min = 9999999;
                $apply_smallest = array();
                $key = '';
                foreach ($costs_array as $k => $v) {
                    if ($v < $min) {
                        $min = $v;
                        $key = $k;
                        $apply_smallest = $k;
                    }
                }

                unset($matched_shipping_methods);
                $matched_shipping_methods[] = $apply_smallest;
            }
        }

        if ($get_what_to_do_method == 'apply_highest') {
            if (!empty($matched_shipping_methods)) {
                foreach ($matched_shipping_methods as $shipping_method_id) {
                    $shipping_title = get_the_title($shipping_method_id);

                    $shipping_rate = array(
                        'id' => $shipping_method_id,
                        'label' => $shipping_title,
                        'cost' => 0,
                    );

                    $costs = get_post_meta($shipping_method_id, 'sm_product_cost', true);
                    $getFeesPerQtyFlag = get_post_meta($shipping_method_id, 'sm_fee_chk_qty_price', true);
                    $getFeesPerQty = get_post_meta($shipping_method_id, 'sm_fee_per_qty', true);
                    $extraProductCostOriginal = get_post_meta($shipping_method_id, 'sm_extra_product_cost', true);

                    $cart_based_qty = '0';
                    $products_based_qty = '0';
                    if ($getFeesPerQtyFlag == 'on') {
                        if ($getFeesPerQty == 'qty_cart_based') {
                            /*Start : Cart Based quantity for Apply Highest*/
                            if (!empty($cart_array)) {
                                foreach ($cart_array as $woo_cart_item_key => $value) {
	                                $product_cart_array = new WC_Product($value['product_id']);
	                                if (!($product_cart_array->is_virtual('yes'))) {
		                                $cart_based_qty += $value['quantity'];
	                                }
                                }
                            }
                            /*End : Cart Based quantity for Apply Highest*/
                        }
                        if ($getFeesPerQty == 'qty_product_based') {
                            /*Start : Product Based quantity for Apply Highest*/
                            $productFeesArray = get_post_meta($shipping_method_id, 'sm_metabox', true);
                            /* @var $productFeesArray type */
                            $check_prd_qty = array();
                            if (!empty($productFeesArray)) {
                                foreach ($productFeesArray as $key => $condition) {
                                    if (array_search('product', $condition)) {
                                        $site_product_id = '';
                                        $cart_final_products_array = array();
                                        /* Product Condition Start */
                                        if ($condition['product_fees_conditions_is'] == 'is_equal_to') {
                                            if (!empty($condition['product_fees_conditions_values'])) {
                                                foreach ($condition['product_fees_conditions_values'] as $product_id) {
                                                    foreach ($cart_array as $key => $value) {
                                                        if (!empty($check_prd_qty)) {
                                                            if (!array_key_exists($value['product_id'], $check_prd_qty)) {
                                                                if (!empty($sitepress)) {
                                                                    $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                                } else {
                                                                    $site_product_id = $value['product_id'];
                                                                }

                                                                if ($product_id == $site_product_id) {
                                                                    $cart_final_products_array[] = $value;
                                                                }
                                                            }
                                                        } else {
                                                            if (!empty($sitepress)) {
                                                                $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                            } else {
                                                                $site_product_id = $value['product_id'];
                                                            }

                                                            if ($product_id == $site_product_id) {
                                                                $cart_final_products_array[] = $value;
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        } elseif ($condition['product_fees_conditions_is'] == 'not_in') {
                                            if (!empty($condition['product_fees_conditions_values'])) {
                                                foreach ($condition['product_fees_conditions_values'] as $product_id) {
                                                    foreach ($cart_array as $key => $value) {
                                                        if (!empty($check_prd_qty)) {
                                                            if (!array_key_exists($value['product_id'], $check_prd_qty)) {
                                                                if (!empty($sitepress)) {
                                                                    $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                                } else {
                                                                    $site_product_id = $value['product_id'];
                                                                }

                                                                if ($product_id != $site_product_id) {
                                                                    $cart_final_products_array[$product_id] = $value;
                                                                }
                                                            }
                                                        } else {
                                                            if (!empty($sitepress)) {
                                                                $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                            } else {
                                                                $site_product_id = $value['product_id'];
                                                            }

                                                            if ($product_id != $site_product_id) {
                                                                $cart_final_products_array[$product_id] = $value;
                                                            }
                                                        }
                                                    }
                                                }
                                            }
                                        }

                                        if (!empty($cart_final_products_array)) {
                                            foreach ($cart_final_products_array as $cart_prd_id => $cart_item) {
	                                            $product_cart_array = new WC_Product($cart_item['product_id']);
	                                            if (!($product_cart_array->is_virtual('yes'))) {
		                                            $products_based_qty += $cart_item['quantity'];
	                                            }
                                            }
                                        }
                                    }
                                }
                            }
                            /*End : Product Based quantity for Apply Highest*/
                        }
                    }

                    // Calculate the costs
                    if (isset($woocommerce_wpml) && !empty($woocommerce_wpml->multi_currency)) {
                        $extraProductCost = $woocommerce_wpml->multi_currency->prices->convert_price_amount($extraProductCostOriginal);
                    } else {
                        $extraProductCost = $extraProductCostOriginal;
                    }

                    if ($getFeesPerQtyFlag == 'on') {
                        if ($getFeesPerQty == 'qty_cart_based') {
                            $costs = $costs + (($cart_based_qty - 1) * $extraProductCost);
                        } elseif ($getFeesPerQty == 'qty_product_based') {
                            $costs = $costs + (($products_based_qty - 1) * $extraProductCost);
                        }
                    }
                    $costs_array[$shipping_method_id] = $costs;
                }
                $max = -9999999;
                $apply_highest = array();
                $key = '';
                foreach ($costs_array as $k => $v) {
                    if ($v > $max) {
                        $max = $v;
                        $key = $k;
                        $apply_highest = $k;
                    }
                }
                unset($matched_shipping_methods);
                $matched_shipping_methods[] = $apply_highest;
            }
        }

        if ($get_what_to_do_method == 'force_all') {
            if (!empty($matched_shipping_methods)) {
                //unset($matched_shipping_methods);
                foreach ($matched_shipping_methods as $shipping_method_id) {
                    $shipping_title = get_the_title($shipping_method_id);

                    $shipping_rate = array(
                        'id' => $shipping_method_id,
                        'label' => $shipping_title,
                        'cost' => 0,
                    );
                    $cart_based_qty = '0';
                    if (!empty($cart_array)) {
                        foreach ($cart_array as $woo_cart_item_key => $value) {
	                        $product_cart_array = new WC_Product($value['product_id']);
	                        if (!($product_cart_array->is_virtual('yes'))) {
		                        $cart_based_qty += $value['quantity'];
	                        }
                        }
                    }

                    // Calculate the costs
                    $has_costs = false; // True when a cost is set. False if all costs are blank strings.
                    $costs = get_post_meta($shipping_method_id, 'sm_product_cost', true);
                    $getFeesPerQtyFlag = get_post_meta($shipping_method_id, 'sm_fee_chk_qty_price', true);
                    $getFeesPerQty = get_post_meta($shipping_method_id, 'sm_fee_per_qty', true);
                    $extraProductCostOriginal = get_post_meta($shipping_method_id, 'sm_extra_product_cost', true);
                    $costs_array[$shipping_method_id] = $costs;
                }
                $forceall = array();
                $total_costs = 0;
                $i = 0;
                $k_with_comma = array();
                foreach ($costs_array as $k => $v) {
                    $key = $k;
                    $forceall_key = 'forceall';
                    $total_costs = $total_costs + $v;
                    $forceall[$i] = $k;
                    $k_with_comma[$i] = $k;
                    $i++;
                }
                $forceall['forceall'] = 0;

                $matched_shipping_methods = $forceall;
            }
        }
        /* End ordering issue and highest, smallest, forceall shipping issue code */
        if (!empty($matched_shipping_methods)) {
            $total_shipping_cost_force_all = 0;  /* ordering issue and highest, smallest, forceall shipping issue code */
            $force_all_shipping_rate = array(); /* ordering issue and highest, smallest, forceall shipping issue code */
            foreach ($matched_shipping_methods as $shipping_method_key => $shipping_method_id_val) {

                $get_condition_array = get_post_meta($shipping_method_id_val, 'sm_metabox', true);

                $shipping_title = get_the_title($shipping_method_id_val);

                $shipping_rate = array(
                    'id' => $shipping_method_id_val,
                    'label' => $shipping_title,
                    'cost' => 0,
                );
                $cart_based_qty = 0;
                if (!empty($cart_array)) {
                    foreach ($cart_array as $woo_cart_item_key => $value) {
	                    $product_cart_array = new WC_Product($value['product_id']);
	                    if (!($product_cart_array->is_virtual('yes'))) {
		                    $cart_based_qty += $value['quantity'];
	                    }
                    }
                }

                // Calculate the costs
                $has_costs = false; // True when a cost is set. False if all costs are blank strings.
                $costs = get_post_meta($shipping_method_id_val, 'sm_product_cost', true);
                $getFeesPerQtyFlag = get_post_meta($shipping_method_id_val, 'sm_fee_chk_qty_price', true);
                $getFeesPerQty = get_post_meta($shipping_method_id_val, 'sm_fee_per_qty', true);
                $extraProductCostOriginal = get_post_meta($shipping_method_id_val, 'sm_extra_product_cost', true);
                $ap_rule_status = get_post_meta($shipping_method_id_val, 'ap_rule_status', true);
                $cost_on_product_status = get_post_meta($shipping_method_id_val, 'cost_on_product_status', true);
                $cost_on_category_status = get_post_meta($shipping_method_id_val, 'cost_on_category_status', true);
                $cost_on_total_cart_qty_status = get_post_meta($shipping_method_id_val, 'cost_on_total_cart_qty_status', true);
                $cost_on_product_weight_status = get_post_meta($shipping_method_id_val, 'cost_on_product_weight_status', true);
                $cost_on_category_weight_status = get_post_meta($shipping_method_id_val, 'cost_on_category_weight_status', true);
                $cost_on_total_cart_weight_status = get_post_meta($shipping_method_id_val, 'cost_on_total_cart_weight_status', true);

                //we can check that if advanced pricing enabled only than it will go for further process
                if ($ap_rule_status == 'on') {
                    //if set advanced Product pricing methods than store in variable as array
                    $get_condition_array_ap_product = get_post_meta($shipping_method_id_val, 'sm_metabox_ap_product', true);
                    //if set advanced Category pricing methods than store in variable as array
                    $get_condition_array_ap_category = get_post_meta($shipping_method_id_val, 'sm_metabox_ap_category', true);
                    //if set advanced total cart qty pricing methods than store in variable as array
                    $get_condition_array_ap_total_cart_qty = get_post_meta($shipping_method_id_val, 'sm_metabox_ap_total_cart_qty', true);
                    //if set advanced product weight pricing methods than store in variable as array
                    $get_condition_array_ap_product_weight = get_post_meta($shipping_method_id_val, 'sm_metabox_ap_product_weight', true);
                    //if set advanced category weight pricing methods than store in variable as array
                    $get_condition_array_ap_category_weight = get_post_meta($shipping_method_id_val, 'sm_metabox_ap_category_weight', true);
                    //if set advanced total cart weight pricing methods than store in variable as array
                    $get_condition_array_ap_total_cart_weight = get_post_meta($shipping_method_id_val, 'sm_metabox_ap_total_cart_weight', true);
                    //if set advanced Class pricing methods than store in variable as array
                    $get_condition_array_ap_class = get_post_meta($shipping_method_id_val, 'sm_metabox_ap_class', true);
                } else {
                    //if no methods created than set null to all variables
                    $get_condition_array_ap_product = '';
                    $get_condition_array_ap_category = '';
                    $get_condition_array_ap_total_cart_qty = '';
                    $get_condition_array_ap_class = '';
                }
                if (isset($woocommerce_wpml) && !empty($woocommerce_wpml->multi_currency)) {
                    $extraProductCost = $woocommerce_wpml->multi_currency->prices->convert_price_amount($extraProductCostOriginal);
                } else {
                    $extraProductCost = $extraProductCostOriginal;
                }
                $products_based_qty = 0;
                $cost = '';
                $before_discount_cost = array();
                $after_discount_cost = array();

                /* Per Qty Condition start */

                //add new condition for apply per quantity only apply if advanced pricing rule disabled
                if ($getFeesPerQtyFlag == 'on' && $ap_rule_status != 'on') {
                    $cart_final_products_array = array();
                    $site_product_id = '';
                    $productFeesArray = get_post_meta($shipping_method_id_val, 'sm_metabox', true);
                    /* @var $productFeesArray type */
                    $check_prd_qty = array();
                    if (!empty($productFeesArray)) {
                        foreach ($productFeesArray as $key => $condition) {
                            if (array_search('product', $condition)) {
                                $site_product_id = '';
                                $cart_final_products_array = array();
                                /* Product Condition Start */
                                if ($condition['product_fees_conditions_is'] == 'is_equal_to') {
                                    if (!empty($condition['product_fees_conditions_values'])) {
                                        foreach ($condition['product_fees_conditions_values'] as $product_id) {
                                            foreach ($cart_array as $key => $value) {
                                                if (!empty($check_prd_qty)) {
                                                    if (!array_key_exists($value['product_id'], $check_prd_qty)) {
                                                        if (!empty($sitepress)) {
                                                            $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                        } else {
                                                            $site_product_id = $value['product_id'];
                                                        }

                                                        if ($product_id == $site_product_id) {
                                                            $cart_final_products_array[] = $value;
                                                        }
                                                    }
                                                } else {
                                                    if (!empty($sitepress)) {
                                                        $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                    } else {
                                                        $site_product_id = $value['product_id'];
                                                    }

                                                    if ($product_id == $site_product_id) {
                                                        $cart_final_products_array[] = $value;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                } elseif ($condition['product_fees_conditions_is'] == 'not_in') {
                                    if (!empty($condition['product_fees_conditions_values'])) {
                                        foreach ($condition['product_fees_conditions_values'] as $product_id) {
                                            foreach ($cart_array as $key => $value) {
                                                if (!empty($check_prd_qty)) {
                                                    if (!array_key_exists($value['product_id'], $check_prd_qty)) {
                                                        if (!empty($sitepress)) {
                                                            $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                        } else {
                                                            $site_product_id = $value['product_id'];
                                                        }

                                                        if ($product_id != $site_product_id) {
                                                            $cart_final_products_array[$product_id] = $value;
                                                        }
                                                    }
                                                } else {
                                                    if (!empty($sitepress)) {
                                                        $site_product_id = apply_filters('wpml_object_id', $value['product_id'], 'product', TRUE, $default_lang);
                                                    } else {
                                                        $site_product_id = $value['product_id'];
                                                    }

                                                    if ($product_id != $site_product_id) {
                                                        $cart_final_products_array[$product_id] = $value;
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }

                                if (!empty($cart_final_products_array)) {
                                    foreach ($cart_final_products_array as $cart_prd_id => $cart_item) {
	                                    $product_cart_array = new WC_Product($cart_item['product_id']);
	                                    if (!($product_cart_array->is_virtual('yes'))) {
		                                    $products_based_qty += $cart_item['quantity'];
		                                    $check_prd_qty[$cart_prd_id] = $cart_item['quantity'];
	                                    }
                                    }
                                }
                                /* Product Condition End */
                            }

                            if (array_search('variableproduct', $condition)) {
                                $site_product_id = '';
                                $cart_final_var_products_array = array();
                                /* Variable Product Condition Start */
                                if ($condition['product_fees_conditions_is'] == 'is_equal_to') {
                                    if (!empty($condition['product_fees_conditions_values'])) {
                                        foreach ($condition['product_fees_conditions_values'] as $product_id) {

                                            foreach ($cart_array as $key => $value) {
                                                if (!empty($sitepress)) {
                                                    $site_product_id = apply_filters('wpml_object_id', $value['variation_id'], 'product', TRUE, $default_lang);
                                                } else {
                                                    $site_product_id = $value['variation_id'];
                                                }

                                                if ($product_id == $site_product_id) {
                                                    $cart_final_var_products_array[] = $value;
                                                }
                                            }
                                        }
                                    }
                                } elseif ($condition['product_fees_conditions_is'] == 'not_in') {
                                    if (!empty($condition['product_fees_conditions_values'])) {
                                        foreach ($condition['product_fees_conditions_values'] as $product_id) {
                                            foreach ($cart_array as $key => $value) {

                                                if (!empty($sitepress)) {
                                                    $site_product_id = apply_filters('wpml_object_id', $value['variation_id'], 'product', TRUE, $default_lang);
                                                } else {
                                                    $site_product_id = $value['product_id'];
                                                }

                                                if ($product_id != $site_product_id) {
                                                    $cart_final_var_products_array[$product_id] = $value;
                                                }
                                            }
                                        }
                                    }
                                }

                                if (!empty($cart_final_var_products_array)) {
                                    foreach ($cart_final_var_products_array as $cart_var_prd_id => $cart_item) {
	                                    $product_cart_array = new WC_Product($cart_item['product_id']);
	                                    if (!($product_cart_array->is_virtual('yes'))) {
		                                    $products_based_qty += $cart_item['quantity'];
		                                    $check_prd_qty[$cart_var_prd_id] = $cart_item['quantity'];
	                                    }
                                    }
                                }
                                /* Variable Product Condition End */
                            }
                            /* Category Condition Start */
                            if (array_search('category', $condition)) {
                                $final_cart_products_cats_ids = array();
                                $cart_final_cat_products_array = array();

                                $all_cats = get_terms(
                                    array(
                                        'taxonomy' => 'product_cat',
                                        'fields' => 'ids'
                                    )
                                );

                                if ($condition['product_fees_conditions_is'] == 'is_equal_to') {
                                    if (!empty($condition['product_fees_conditions_values'])) {
                                        foreach ($condition['product_fees_conditions_values'] as $category_id) {
                                            $final_cart_products_cats_ids[] = $category_id;
                                        }
                                    }
                                } elseif ($condition['product_fees_conditions_is'] == 'not_in') {
                                    if (!empty($condition['product_fees_conditions_values'])) {
                                        $final_cart_products_cats_ids = array_diff($all_cats, $condition['product_fees_conditions_values']);
                                    }
                                }

                                $cat_args = array(
                                    'post_type' => 'product',
                                    'posts_per_page' => -1,
                                    'order' => 'ASC',
                                    'fields' => 'ids',
                                    'tax_query' => array(
                                        array(
                                            'taxonomy' => 'product_cat',
                                            'field' => 'term_id',
                                            'terms' => $final_cart_products_cats_ids,
                                        )
                                    )
                                );
                                $cat_products_ids = get_posts($cat_args);

                                foreach ($cart_array as $key => $value) {
                                    if (in_array($value['product_id'], $cat_products_ids)) {
                                        if (!empty($check_prd_qty)) {
                                            if (!array_key_exists($value['product_id'], $check_prd_qty)) {
                                                $cart_final_cat_products_array[$value['product_id']] = $value;
                                            }
                                        } else {
                                            $cart_final_cat_products_array[$value['product_id']] = $value;
                                        }
                                    }
                                }

                                if (!empty($cart_final_cat_products_array)) {
                                    foreach ($cart_final_cat_products_array as $cart_cat_prd_id => $cart_item) {
	                                    $product_cart_array = new WC_Product($cart_item['product_id']);
	                                    if (!($product_cart_array->is_virtual('yes'))) {
		                                    $products_based_qty += $cart_item['quantity'];
		                                    $check_prd_qty[$cart_cat_prd_id] = $cart_item['quantity'];
	                                    }
                                    }
                                }
                            }
                            /* Category Condition End */
                            if (array_search('tag', $condition)) {

                                /* Tag Condition Start */
                                $final_cart_products_tag_ids = array();
                                $cart_final_tag_products_array = array();
                                $final_cart_products_tag_not_in_flag = 0;
                                $all_tags = get_terms(
                                    array(
                                        'taxonomy' => 'product_tag',
                                        'fields' => 'ids'
                                    )
                                );
                                if ($condition['product_fees_conditions_is'] == 'is_equal_to') {
                                    if (!empty($condition['product_fees_conditions_values'])) {
                                        foreach ($condition['product_fees_conditions_values'] as $tag_id) {
                                            $final_cart_products_tag_ids[] = $tag_id;
                                        }
                                    }
                                } elseif ($condition['product_fees_conditions_is'] == 'not_in') {
                                    if (!empty($condition['product_fees_conditions_values'])) {
                                        $final_cart_products_tag_not_in_flag = 1;
                                        $final_cart_products_tag_ids = array_diff($all_tags, $condition['product_fees_conditions_values']);
                                    }
                                }

                                $tag_args = array(
                                    'post_type' => 'product',
                                    'posts_per_page' => -1,
                                    'order' => 'ASC',
                                    'fields' => 'ids',
                                    'tax_query' => array(
                                        array(
                                            'taxonomy' => 'product_tag',
                                            'field' => 'term_id',
                                            'terms' => $final_cart_products_tag_ids,
                                        )
                                    )
                                );
                                $tag_products_ids = get_posts($tag_args);

                                foreach ($cart_array as $key => $value) {
                                    if (!empty($check_prd_qty)) {
                                        if (!array_key_exists($value['product_id'], $check_prd_qty)) {
                                            if ($final_cart_products_tag_not_in_flag == 1) {
                                                $cart_final_tag_products_array[$value['product_id']] = $value;
                                            } elseif (in_array($value['product_id'], $tag_products_ids)) {
                                                $cart_final_tag_products_array[$value['product_id']] = $value;
                                            }
                                        }
                                    } else {
                                        if ($final_cart_products_tag_not_in_flag == 1) {
                                            $cart_final_tag_products_array[$value['product_id']] = $value;
                                        } elseif (in_array($value['product_id'], $tag_products_ids)) {
                                            $cart_final_tag_products_array[$value['product_id']] = $value;
                                        }
                                    }
                                }
                                if (!empty($cart_final_tag_products_array)) {
                                    foreach ($cart_final_tag_products_array as $cart_tag_prd_id => $cart_item) {
	                                    $product_cart_array = new WC_Product($cart_item['product_id']);
	                                    if (!($product_cart_array->is_virtual('yes'))) {
		                                    $products_based_qty += $cart_item['quantity'];
		                                    $check_prd_qty[$cart_tag_prd_id] = $cart_item['quantity'];
	                                    }
                                    }
                                }
                            }
                            /* Tag Condition End */
                        }
                    }

                    if ($getFeesPerQty == 'qty_cart_based') {
                        $cost = $costs + (($cart_based_qty - 1) * $extraProductCost);
                    } else if ($getFeesPerQty == 'qty_product_based') {
                        $cost = $costs + (($products_based_qty - 1) * $extraProductCost);
                    }
                    /* Per Qty Condition end */
                } else {
                    $cost = $costs;
                    if (!empty($get_condition_array)) {
                        $cart_total_array = array();
                        $cart_totalafter_array = array();
                        foreach ($get_condition_array as $key => $value) {
                            if (array_search('cart_total', $value)) {
                                $cart_total_array[$key] = $value;
                                $total = WC()->cart->subtotal;
                                if (isset($woocommerce_wpml) && !empty($woocommerce_wpml->multi_currency)) {
                                    $new_total = $woocommerce_wpml->multi_currency->prices->unconvert_price_amount($total);
                                } else {
                                    $new_total = $total;
                                }

                                if ($value['product_fees_conditions_is'] == 'is_equal_to' ||
                                    $value['product_fees_conditions_is'] == 'less_equal_to' ||
                                    $value['product_fees_conditions_is'] == 'less_then' ||
                                    $value['product_fees_conditions_is'] == 'greater_equal_to' ||
                                    $value['product_fees_conditions_is'] == 'greater_then' ||
                                    $value['product_fees_conditions_is'] == 'not_in') {
                                    if (!empty($value['product_fees_conditions_values'])) {
                                        if ($value['product_fees_conditions_values'] == $new_total ||
                                            $value['product_fees_conditions_values'] >= $new_total ||
                                            $value['product_fees_conditions_values'] > $new_total ||
                                            $value['product_fees_conditions_values'] <= $new_total ||
                                            $value['product_fees_conditions_values'] < $new_total ||
                                            $new_total == $value['product_fees_conditions_values']) {

                                            $cost_args = array(
                                                'qty' => $this->get_package_item_qty($package),
                                                'cost' => $new_total
                                            );
                                            $before_discount_cost['before'] = $cost_args;
                                        }
                                    }
                                }
                            }

                            if (array_search('cart_totalafter', $value)) {
                                $cart_totalafter_array[$key] = $value;

                                $admin_object = new Advanced_Flat_Rate_Shipping_For_WooCommerce_Pro_Admin('', '');
                                $totalprice = $admin_object->afrsm_pro_remove_currency_symbol($woocommerce->cart->get_cart_subtotal());
                                $totaldisc = $admin_object->afrsm_pro_remove_currency_symbol($woocommerce->cart->get_total_discount());
                                if ($totaldisc != '' || $totaldisc != '0') {
                                    $resultprice = $totalprice - $totaldisc;
                                    if (isset($woocommerce_wpml) && !empty($woocommerce_wpml->multi_currency)) {
                                        $new_resultprice = $woocommerce_wpml->multi_currency->prices->unconvert_price_amount($resultprice);
                                    } else {
                                        $new_resultprice = $resultprice;
                                    }
                                    if ($value['product_fees_conditions_is'] == 'is_equal_to' ||
                                        $value['product_fees_conditions_is'] == 'less_equal_to' ||
                                        $value['product_fees_conditions_is'] == 'less_then' ||
                                        $value['product_fees_conditions_is'] == 'greater_equal_to' ||
                                        $value['product_fees_conditions_is'] == 'greater_then' ||
                                        $value['product_fees_conditions_is'] == 'not_in') {
                                        if ($value['product_fees_conditions_values'] == $new_resultprice ||
                                            $value['product_fees_conditions_values'] >= $new_resultprice ||
                                            $value['product_fees_conditions_values'] > $new_resultprice ||
                                            $value['product_fees_conditions_values'] <= $new_resultprice ||
                                            $value['product_fees_conditions_values'] < $new_resultprice ||
                                            $new_resultprice == $value['product_fees_conditions_values']) {

                                            $cost_args = array(
                                                'qty' => $this->get_package_item_qty($package),
                                                'cost' => $new_resultprice
                                            );
                                            $after_discount_cost['after'] = $cost_args;
                                        }
                                    }
                                }
                            }
                        }
                    }

                }

                $sm_taxable = get_post_meta($shipping_method_id_val, 'sm_select_taxable', true);
                $sm_extra_cost_calculation_type = get_post_meta($shipping_method_id_val, 'sm_extra_cost_calculation_type', true);

                if ($cost !== '') {
                    $has_costs = true;
                    if (!empty($before_discount_cost)) {
                        if (array_key_exists('before', $before_discount_cost)) {
                            $shipping_rate['cost'] = $this->evaluate_cost($cost, $before_discount_cost['before']);
                        }
                    } else if (!empty($after_discount_cost)) {
                        if (array_key_exists('after', $after_discount_cost)) {
                            $shipping_rate['cost'] = $this->evaluate_cost($cost, $after_discount_cost['after']);
                        }
                    } else {
                        $cost_args = array(
                            'qty' => $this->get_package_item_qty($package),
                            'cost' => $package['contents_cost']
                        );
                        $shipping_rate['cost'] = $this->evaluate_cost($cost, $cost_args);
                    }
                }

                // Add shipping class costs
                $found_shipping_classes = $this->find_shipping_classes($package);
                $highest_class_cost = 0;

                if (!empty($found_shipping_classes)) {
                    foreach ($found_shipping_classes as $shipping_class => $products) {
                        $shipping_class_term = get_term_by('slug', $shipping_class, 'product_shipping_class');
                        $shipping_extra_id = '';
                        if ($shipping_class_term !== FALSE) {

                            if (!empty($sitepress)) {
                                $shipping_extra_id = apply_filters('wpml_object_id', $shipping_class_term->term_id, 'product_shipping_class', TRUE, $default_lang);
                            } else {
                                $shipping_extra_id = $shipping_class_term->term_id;
                            }
                        }
                        $sm_extra_cost = get_post_meta($shipping_method_id_val, 'sm_extra_cost', true);
                        $class_cost_string = isset($sm_extra_cost[$shipping_extra_id]) && !empty($sm_extra_cost[$shipping_extra_id]) ? $sm_extra_cost[$shipping_extra_id] : '';

                        if ($class_cost_string === '') {
                            continue;
                        }

                        $has_costs = true;
                        $class_cost = $this->evaluate_cost($class_cost_string, array(
                            'qty' => array_sum(wp_list_pluck($products, 'quantity')),
                            'cost' => array_sum(wp_list_pluck($products, 'line_total'))
                        ));

                        if ($sm_extra_cost_calculation_type === 'per_class') {
                            $shipping_rate['cost'] += $class_cost;
                        } else {
                            $highest_class_cost = $class_cost > $highest_class_cost ? $class_cost : $highest_class_cost;
                        }
                    }
                    if ($sm_extra_cost_calculation_type === 'per_order' && $highest_class_cost) {
                        $shipping_rate['cost'] += $highest_class_cost;
                    }
                }

                if ($sm_taxable == 'no') {
                    $shipping_rate['taxes'] = false;
                } else {
                    $shipping_rate['taxes'] = '';
                }

                /* Apply Advanced Pricing rules Cost start here */

                //if advanced pricing for product exist
                if ($cost_on_product_status == 'on') {
                    if (!empty($get_condition_array_ap_product)) {
                        //create variable for store all valid product rule as array
                        $product_array_ap = array();
                        foreach ($get_condition_array_ap_product as $key => $value) {
                            //store rules in array if product of AP rule selected
                            if ($value['ap_fees_products'] != '')
                                $product_array_ap[$key] = $value; //store all valid rule in array
                        }
                        //go inside if rules set found
                        if (is_array($product_array_ap) && isset($product_array_ap) && !empty($product_array_ap) && !empty($cart_array)) {

                            //get cart data in variable
                            $woo_cart_array = $cart_array;
                            //get cart total items for percent calculation
                            $get_cart_total = WC()->cart->get_cart_contents_total();

                            //initialize array for store all product IDs of rule set
                            $cart_products_array = array();

                            //get all product column and store in array for further use
                            $cart_product = $this->fee_array_column($cart_array, 'product_id');

                            //if product rule found with product id
                            if (isset($cart_product) && !empty($cart_product)) {
                                foreach ($cart_product as $key => $cart_product_id) {
                                    //store all product ids in array
                                    if (!empty($sitepress)) {
                                        $cart_products_array[] = apply_filters('wpml_object_id', $cart_product_id, 'product', TRUE, $default_lang);
                                    } else {
                                        $cart_products_array[] = $cart_product_id;
                                    }
                                }
                            }
                            foreach ($product_array_ap as $product) {

                                $quantity_total = 0;
                                $ap_selected_product_id = $product['ap_fees_products'];

                                $prd_id_list = array();
                                foreach ($woo_cart_array as $woo_cart_item_key => $woo_cart_item) {
                                    if (!empty($sitepress)) {
                                        $product_id_lan = apply_filters('wpml_object_id', $woo_cart_item['product_id'], 'product', TRUE, $default_lang);
                                    } else {
                                        $product_id_lan = $woo_cart_item['product_id'];
                                    }

                                    //get product rule set total cart quantity for further use in condition for selected product only
                                    if (in_array($product_id_lan, $ap_selected_product_id)) {
	                                    $product_cart_array = new WC_Product($woo_cart_item['product_id']);
	                                    if (!($product_cart_array->is_virtual('yes'))) {
		                                    $quantity_total += $woo_cart_item['quantity']; //increase category as quantity in item
	                                    }
                                    }
                                }

                                //store all array key to variable
                                $ap_fees_ap_min_qty = $product['ap_fees_ap_prd_min_qty'];
                                $ap_fees_ap_max_qty = $product['ap_fees_ap_prd_max_qty'];
                                $ap_fees_ap_price_product = $product['ap_fees_ap_price_product'];
                                $ap_fees_products = $product['ap_fees_products'];

                                //check if quantity min or max exist and cost greater than 0 than it will go for further process
                                if (($ap_fees_ap_min_qty != '' || $ap_fees_ap_max_qty != '') && $ap_fees_ap_price_product != 0) {

                                    $product_id = $ap_fees_products;
                                    $has_product = false;
                                    foreach ($ap_fees_products as $product_id) {
                                        if (in_array($product_id, $cart_products_array)) {
                                            $has_product = true;
                                        }
                                    }
                                    if ($has_product == true) {
                                        //we testing all Advanced pricing method rules here
                                        //if true than add price to shipping method
                                        $ap_product_rule = 0;
                                        if (($quantity_total >= $ap_fees_ap_min_qty || $ap_fees_ap_min_qty == '') &&
                                            ($quantity_total <= $ap_fees_ap_max_qty || $ap_fees_ap_max_qty == '')) {
                                            //set flag 1 if condition matched
                                            $ap_product_rule = 1;
                                        }
                                        //if rule set condition match true/1 than append rule cost in shipping method's cost
                                        if ($ap_product_rule == 1) {
                                            $is_percent = substr($ap_fees_ap_price_product, -1); // returns "last character"
                                            if ($is_percent == '%') { //if price in percent than go inside
                                                //get price without sign
                                                $percent = substr($ap_fees_ap_price_product, 0, -1);
                                                //convert price in to float format
                                                $percent = number_format((float)$percent, 2, '.', '');
                                                if (!empty($percent)) {
                                                    //add cost to shipping method
                                                    $shipping_rate['cost'] = $shipping_rate['cost'] + ($percent / 100) * $get_cart_total;
                                                }
                                            } else {
                                                $shipping_rate['cost'] = $shipping_rate['cost'] + $ap_fees_ap_price_product;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                //if advanced pricing for Category exist
                if ($cost_on_category_status == 'on') {
                    if (!empty($get_condition_array_ap_category)) {
                        //initialize array
                        $category_array_ap = array();
                        foreach ($get_condition_array_ap_category as $key => $value) { //store rules as array
                            if ($value['ap_fees_categories'] != '')
                                $category_array_ap[$key] = $value; //store all valid rule in array
                        }
                        //go inside if valid rules found
                        if (is_array($category_array_ap) && isset($category_array_ap) && !empty($category_array_ap) && !empty($cart_array)) {
                            //get cart data in variable
                            $woo_cart_array = $cart_array;
                            //get cart total items for percent calculation
                            $get_cart_total = WC()->cart->get_cart_contents_total();
                            foreach ($category_array_ap as $category) {
                                //initialize category count
                                $quantity_cat_total = 0;
                                //get rule category id
                                $ap_selected_categories_id = $category['ap_fees_categories'];

                                foreach ($woo_cart_array as $woo_cart_item_key => $woo_cart_item) { //cart loop
                                    //get current product total categories id
                                    if (!empty($sitepress)) {
                                        $post_lan_id = apply_filters('wpml_object_id', $woo_cart_item['product_id'], 'product', TRUE, $default_lang);
                                    } else {
                                        $post_lan_id = $woo_cart_item['product_id'];
                                    }
                                    $cat_id_list = wp_get_post_terms($post_lan_id, 'product_cat', array('fields' => 'ids'));

                                    //initialize language array
                                    $cat_id_list_origin = array();
                                    if (isset($cat_id_list) && !empty($cat_id_list)) {
                                        foreach ($cat_id_list as $key => $cat_id) {
                                            //store all product ids in array
                                            if (!empty($sitepress)) {
                                                $cat_id_list_origin[] = apply_filters('wpml_object_id', $cat_id, 'product_cat', TRUE, $default_lang);
                                            } else {
                                                $cat_id_list_origin[] = $cat_id;
                                            }
                                        }
                                    }

                                    //check if cat list empty or not
                                    if (!empty($cat_id_list_origin) && is_array($cat_id_list_origin)) {
                                        //check if rule category exist in current cart item than increase the cat quantity
                                        foreach ($ap_selected_categories_id as $ap_fees_categories_key => $ap_fees_categories_key_val) {
                                            if (in_array($ap_fees_categories_key_val, $cat_id_list_origin)) {
	                                            $product_cart_array = new WC_Product($woo_cart_item['product_id']);
	                                            if (!($product_cart_array->is_virtual('yes'))) {
		                                            $quantity_cat_total += $woo_cart_item['quantity']; //increase category as quantity in item
	                                            }
                                            }
                                        }
                                    }
                                }

                                //store all array key to variable
                                $ap_fees_ap_cat_min_qty = $category['ap_fees_ap_cat_min_qty'];
                                $ap_fees_ap_cat_max_qty = $category['ap_fees_ap_cat_max_qty'];
                                $ap_fees_ap_price_category = $category['ap_fees_ap_price_category'];
                                $ap_fees_categories = $category['ap_fees_categories'];

                                //check if quantity min or max exist and cost greater than 0 than it will go for further process
                                if (
                                    ($ap_fees_ap_cat_min_qty != '' || $ap_fees_ap_cat_max_qty != '') && $ap_fees_ap_price_category != 0 && $ap_fees_categories != '') {

                                    //check AP Category rule match or not
                                    $ap_category_rule_valid = 0;
                                    if ($quantity_cat_total > 0) {
                                        if (($quantity_cat_total >= $ap_fees_ap_cat_min_qty || $ap_fees_ap_cat_min_qty == '') &&
                                            ($quantity_cat_total <= $ap_fees_ap_cat_max_qty || $ap_fees_ap_cat_max_qty == '')) {
                                            $ap_category_rule_valid = 1;
                                        }
                                        //if set condition match true/1 than append rule cost in shipping method's cost
                                        if ($ap_category_rule_valid == 1) {

                                            $is_percent = substr($ap_fees_ap_price_category, -1); // returns "last character"
                                            if ($is_percent == '%') {
                                                $percent = substr($ap_fees_ap_price_category, 0, -1);
                                                $percent = number_format((float)$percent, 2, '.', '');
                                                if (!empty($percent)) {
                                                    //add percent price to shipping cost
                                                    $shipping_rate['cost'] = $shipping_rate['cost'] + ($percent / 100) * $get_cart_total;
                                                }
                                            } else {
                                                //add normal price to shipping cost.
                                                $shipping_rate['cost'] = $shipping_rate['cost'] + $ap_fees_ap_price_category;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                //if advanced pricing for Total Cart Qty exist
                if ($cost_on_total_cart_qty_status == 'on') {
                    if (!empty($get_condition_array_ap_total_cart_qty)) {
                        //initialize array
                        $total_cart_qty_array_ap = array();
                        foreach ($get_condition_array_ap_total_cart_qty as $key => $value) { //store rules as array
                            if ($value['ap_fees_total_cart_qty'] != '')
                                $total_cart_qty_array_ap[$key] = $value; //store all valid rule in array
                        }
                        //go inside if valid rules found
                        if (is_array($total_cart_qty_array_ap) && isset($total_cart_qty_array_ap) && !empty($total_cart_qty_array_ap) && !empty($cart_array)) {
                            //get cart data in variable
                            $woo_cart_array = $cart_array;
                            //get cart total items for percent calculation
                            $get_cart_total = WC()->cart->get_cart_contents_total();
                            foreach ($total_cart_qty_array_ap as $total_cart_qty) {
                                //initialize total_cart_qty count
                                $quantity_total_cart_qty_total = 0;
                                //get rule total_cart_qty
                                foreach ($woo_cart_array as $woo_cart_item_key => $woo_cart_item) { //cart loop
	                                $product_cart_array = new WC_Product($woo_cart_item['product_id']);
	                                if (!($product_cart_array->is_virtual('yes'))) {
		                                $quantity_total_cart_qty_total += $woo_cart_item['quantity']; //increase total_cart_qty as quantity in item
	                                }
                                }

                                //store all array key to variable
                                $ap_fees_ap_total_cart_qty_min_qty = $total_cart_qty['ap_fees_ap_total_cart_qty_min_qty'];
                                $ap_fees_ap_total_cart_qty_max_qty = $total_cart_qty['ap_fees_ap_total_cart_qty_max_qty'];
                                $ap_fees_ap_price_total_cart_qty = $total_cart_qty['ap_fees_ap_price_total_cart_qty'];
                                $ap_fees_total_cart_qty = $total_cart_qty['ap_fees_total_cart_qty'];

                                //check if quantity min or max exist and cost greater than 0 than it will go for further process
                                if (
                                    ($ap_fees_ap_total_cart_qty_min_qty != '' || $ap_fees_ap_total_cart_qty_max_qty != '') && $ap_fees_ap_price_total_cart_qty != 0 && $ap_fees_total_cart_qty != '') {

                                    //check AP Category rule match or not
                                    $ap_total_cart_qty_rule_valid = 0;
                                    if ($quantity_total_cart_qty_total > 0) {
                                        if (($quantity_total_cart_qty_total >= $ap_fees_ap_total_cart_qty_min_qty || $ap_fees_ap_total_cart_qty_min_qty == '') &&
                                            ($quantity_total_cart_qty_total <= $ap_fees_ap_total_cart_qty_max_qty || $ap_fees_ap_total_cart_qty_max_qty == '')) {
                                            $ap_total_cart_qty_rule_valid = 1;
                                        }

                                        //if set condition match true/1 than append rule cost in shipping method's cost
                                        if ($ap_total_cart_qty_rule_valid == 1) {
                                            $is_percent = substr($ap_fees_ap_price_total_cart_qty, -1); // returns "last character"
                                            if ($is_percent == '%') {
                                                $percent = substr($ap_fees_ap_price_total_cart_qty, 0, -1);
                                                $percent = number_format((float)$percent, 2, '.', '');
                                                if (!empty($percent)) {
                                                    //add percent price to shipping cost
                                                    $shipping_rate['cost'] = $shipping_rate['cost'] + ($percent / 100) * $get_cart_total;
                                                }
                                            } else {
                                                //add normal price to shipping cost.
                                                $shipping_rate['cost'] = $shipping_rate['cost'] + $ap_fees_ap_price_total_cart_qty;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                //if advanced pricing for product exist
                if ($cost_on_product_weight_status == 'on') {
                    if (!empty($get_condition_array_ap_product_weight)) {
                        //create variable for store all valid product rule as array
                        $product_weight_array_ap = array();
                        foreach ($get_condition_array_ap_product_weight as $key => $value) {
                            //store rules in array if product of AP rule selected
                            if ($value['ap_fees_product_weight'] != '')
                                $product_weight_array_ap[$key] = $value; //store all valid rule in array
                        }
                        //go inside if rules set found
                        if (is_array($product_weight_array_ap) && isset($product_weight_array_ap) && !empty($product_weight_array_ap) && !empty($cart_array)) {

                            //get cart data in variable
                            $woo_cart_array = $cart_array;
                            //get cart total items for percent calculation
                            $get_cart_total = WC()->cart->get_cart_contents_total();

                            //initialize array for store all product IDs of rule set
                            $cart_products_array = array();

                            //get all product column and store in array for further use
                            $cart_product = $this->fee_array_column($cart_array, 'product_id');

                            //if product rule found with product id
                            if (isset($cart_product) && !empty($cart_product)) {
                                foreach ($cart_product as $key => $cart_product_id) {
                                    //store all product ids in array
                                    if (!empty($sitepress)) {
                                        $cart_products_array[] = apply_filters('wpml_object_id', $cart_product_id, 'product', TRUE, $default_lang);
                                    } else {
                                        $cart_products_array[] = $cart_product_id;
                                    }
                                }
                            }
                            foreach ($product_weight_array_ap as $product) {

                                $weight_total = 0;
                                $ap_selected_product_weight_id = $product['ap_fees_product_weight'];

                                $prd_id_list = array();
                                foreach ($woo_cart_array as $woo_cart_item_key => $woo_cart_item) {
                                    if (!empty($sitepress)) {
                                        $product_id_lan = apply_filters('wpml_object_id', $woo_cart_item['product_id'], 'product', TRUE, $default_lang);
                                    } else {
                                        $product_id_lan = $woo_cart_item['product_id'];
                                    }

                                    //get product rule set total cart quantity for further use in condition for selected product only
                                    if (in_array($product_id_lan, $ap_selected_product_weight_id)) {
	                                    $product_cart_array = new WC_Product($woo_cart_item['product_id']);
	                                    if (!($product_cart_array->is_virtual('yes'))) {
		                                    $get_product  = wc_get_product( $woo_cart_item['product_id'] );
		                                    $weight_total += (int) $woo_cart_item['quantity'] * (float) $get_product->get_weight();
	                                    }
                                    }
                                }

                                //store all array key to variable
                                $ap_fees_ap_product_weight_min_qty = $product['ap_fees_ap_product_weight_min_qty'];
                                $ap_fees_ap_product_weight_max_qty = $product['ap_fees_ap_product_weight_max_qty'];
                                $ap_fees_ap_product_weight_price_product = $product['ap_fees_ap_price_product_weight'];
                                $ap_fees_product_weight = $product['ap_fees_product_weight'];

                                //check if quantity min or max exist and cost greater than 0 than it will go for further process
                                if (($ap_fees_ap_product_weight_min_qty != '' || $ap_fees_ap_product_weight_max_qty != '') && $ap_fees_ap_product_weight_price_product != 0) {

                                    $product_id = $ap_fees_product_weight;
                                    $has_product = false;
                                    foreach ($ap_fees_product_weight as $product_id) {
                                        if (in_array($product_id, $cart_products_array)) {
                                            $has_product = true;
                                        }
                                    }
                                    if ($has_product == true) {
                                        //we testing all Advanced pricing method rules here
                                        //if true than add price to shipping method
                                        $ap_product_rule = 0;
                                        if (($weight_total >= $ap_fees_ap_product_weight_min_qty || $ap_fees_ap_product_weight_min_qty == '') &&
                                            ($weight_total <= $ap_fees_ap_product_weight_max_qty || $ap_fees_ap_product_weight_max_qty == '')) {
                                            //set flag 1 if condition matched
                                            $ap_product_rule = 1;
                                        }
                                        //if rule set condition match true/1 than append rule cost in shipping method's cost
                                        if ($ap_product_rule == 1) {
                                            $is_percent = substr($ap_fees_ap_product_weight_price_product, -1); // returns "last character"
                                            if ($is_percent == '%') { //if price in percent than go inside
                                                //get price without sign
                                                $percent = substr($ap_fees_ap_product_weight_price_product, 0, -1);
                                                //convert price in to float format
                                                $percent = number_format((float)$percent, 2, '.', '');
                                                if (!empty($percent)) {
                                                    //add cost to shipping method
                                                    $shipping_rate['cost'] = $shipping_rate['cost'] + ($percent / 100) * $get_cart_total;
                                                }
                                            } else {
                                                $shipping_rate['cost'] = $shipping_rate['cost'] + $ap_fees_ap_product_weight_price_product;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                //if advanced pricing for Category weight exist
                if ($cost_on_category_weight_status == 'on') {
                    if (!empty($get_condition_array_ap_category_weight)) {
                        //initialize array
                        $category_weight_array_ap = array();
                        foreach ($get_condition_array_ap_category_weight as $key => $value) { //store rules as array
                            if ($value['ap_fees_categories_weight'] != '')
                                $category_weight_array_ap[$key] = $value; //store all valid rule in array
                        }
                        //go inside if valid rules found
                        if (is_array($category_weight_array_ap) && isset($category_weight_array_ap) && !empty($category_weight_array_ap) && !empty($cart_array)) {
                            //get cart data in variable
                            $woo_cart_array = $cart_array;
                            //get cart total items for percent calculation
                            $get_cart_total = WC()->cart->get_cart_contents_total();
                            foreach ($category_weight_array_ap as $category_weight) {
                                //initialize category_weight count
                                $category_weight_total = 0;
                                //get rule category_weight id
                                $ap_selected_categories_weight_id = $category_weight['ap_fees_categories_weight'];

                                foreach ($woo_cart_array as $woo_cart_item_key => $woo_cart_item) { //cart loop
                                    //get current product total categories_weight id
                                    if (!empty($sitepress)) {
                                        $post_lan_id = apply_filters('wpml_object_id', $woo_cart_item['product_id'], 'product', TRUE, $default_lang);
                                    } else {
                                        $post_lan_id = $woo_cart_item['product_id'];
                                    }
                                    $cat_id_list = wp_get_post_terms($post_lan_id, 'product_cat', array('fields' => 'ids'));

                                    //initialize language array
                                    $cat_id_list_origin = array();
                                    if (isset($cat_id_list) && !empty($cat_id_list)) {
                                        foreach ($cat_id_list as $key => $cat_id) {
                                            //store all product ids in array
                                            if (!empty($sitepress)) {
                                                $cat_id_list_origin[] = apply_filters('wpml_object_id', $cat_id, 'product_cat', TRUE, $default_lang);
                                            } else {
                                                $cat_id_list_origin[] = $cat_id;
                                            }
                                        }
                                    }

                                    //check if cat list empty or not
                                    if (!empty($cat_id_list_origin) && is_array($cat_id_list_origin)) {
                                        //check if rule category_weight exist in current cart item than increase the cat quantity
                                        foreach ($ap_selected_categories_weight_id as $ap_fees_categories_weight_key => $ap_fees_categories_weight_key_val) {
                                            if (in_array($ap_fees_categories_weight_key_val, $cat_id_list_origin)) {
                                                //$category_weight_total += $woo_cart_item['quantity']; //increase category_weight as quantity in item
                                                $get_product = wc_get_product($woo_cart_item['product_id']);
                                                $category_weight_total += (int)$woo_cart_item['quantity'] * (float)$get_product->get_weight();
                                            }
                                        }
                                    }
                                }

                                //store all array key to variable
                                $ap_fees_ap_category_weight_min_qty = $category_weight['ap_fees_ap_category_weight_min_qty'];
                                $ap_fees_ap_category_weight_max_qty = $category_weight['ap_fees_ap_category_weight_max_qty'];
                                $ap_fees_ap_price_category_weight = $category_weight['ap_fees_ap_price_category_weight'];
                                $ap_fees_categories_weight = $category_weight['ap_fees_categories_weight'];

                                //check if quantity min or max exist and cost greater than 0 than it will go for further process
                                if (
                                    ($ap_fees_ap_category_weight_min_qty != '' || $ap_fees_ap_category_weight_max_qty != '') && $ap_fees_ap_price_category_weight != 0 && $ap_fees_categories_weight != '') {

                                    //check AP Category rule match or not
                                    $ap_category_weight_rule_valid = 0;
                                    if ($category_weight_total > 0) {
                                        if (($category_weight_total >= $ap_fees_ap_category_weight_min_qty || $ap_fees_ap_category_weight_min_qty == '') &&
                                            ($category_weight_total <= $ap_fees_ap_category_weight_max_qty || $ap_fees_ap_category_weight_max_qty == '')) {
                                            $ap_category_weight_rule_valid = 1;
                                        }
                                        //if set condition match true/1 than append rule cost in shipping method's cost
                                        if ($ap_category_weight_rule_valid == 1) {

                                            $is_percent = substr($ap_fees_ap_price_category_weight, -1); // returns "last character"
                                            if ($is_percent == '%') {
                                                $percent = substr($ap_fees_ap_price_category_weight, 0, -1);
                                                $percent = number_format((float)$percent, 2, '.', '');
                                                if (!empty($percent)) {
                                                    //add percent price to shipping cost
                                                    $shipping_rate['cost'] = $shipping_rate['cost'] + ($percent / 100) * $get_cart_total;
                                                }
                                            } else {
                                                //add normal price to shipping cost.
                                                $shipping_rate['cost'] = $shipping_rate['cost'] + $ap_fees_ap_price_category_weight;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                //if advanced pricing for Total Cart Weight exist
                if ($cost_on_total_cart_weight_status == 'on') {
                    if (!empty($get_condition_array_ap_total_cart_weight)) {
                        //initialize array
                        $total_cart_weight_array_ap = array();
                        foreach ($get_condition_array_ap_total_cart_weight as $key => $value) { //store rules as array
                            if ($value['ap_fees_total_cart_weight'] != '')
                                $total_cart_weight_array_ap[$key] = $value; //store all valid rule in array
                        }
                        //go inside if valid rules found
                        if (is_array($total_cart_weight_array_ap) && isset($total_cart_weight_array_ap) && !empty($total_cart_weight_array_ap) && !empty($cart_array)) {
                            //get cart data in variable
                            $woo_cart_array = $cart_array;
                            //get cart total items for percent calculation
                            $get_cart_total = WC()->cart->get_cart_contents_total();
                            foreach ($total_cart_weight_array_ap as $total_cart_weight) {
                                //initialize total_cart_weight count
                                $quantity_total_cart_weight_total = 0;
                                //get rule total_cart_weight
                                foreach ($woo_cart_array as $woo_cart_item_key => $woo_cart_item) { //cart loop
                                    //$quantity_total_cart_weight_total += $woo_cart_item['quantity']; //increase total_cart_weight as quantity in item
                                    $get_product_weight = wc_get_product($woo_cart_item['product_id']);
	                                $product_cart_array = new WC_Product($woo_cart_item['product_id']);
	                                if (!($product_cart_array->is_virtual('yes'))) {
		                                $quantity_total_cart_weight_total += (int)$woo_cart_item['quantity'] * (float)$get_product_weight->get_weight();
	                                }
                                }

                                //store all array key to variable
                                $ap_fees_ap_total_cart_weight_min_qty = $total_cart_weight['ap_fees_ap_total_cart_weight_min_weight'];
                                $ap_fees_ap_total_cart_weight_max_qty = $total_cart_weight['ap_fees_ap_total_cart_weight_max_weight'];
                                $ap_fees_ap_price_total_cart_weight = $total_cart_weight['ap_fees_ap_price_total_cart_weight'];
                                $ap_fees_total_cart_weight = $total_cart_weight['ap_fees_total_cart_weight'];

                                //check if quantity min or max exist and cost greater than 0 than it will go for further process
                                if (
                                    ($ap_fees_ap_total_cart_weight_min_qty != '' || $ap_fees_ap_total_cart_weight_max_qty != '') && $ap_fees_ap_price_total_cart_weight != 0 && $ap_fees_total_cart_weight != '') {

                                    //check AP Category rule match or not
                                    $ap_total_cart_weight_rule_valid = 0;
                                    if ($quantity_total_cart_weight_total > 0) {
                                        if (($quantity_total_cart_weight_total >= $ap_fees_ap_total_cart_weight_min_qty || $ap_fees_ap_total_cart_weight_min_qty == '') &&
                                            ($quantity_total_cart_weight_total <= $ap_fees_ap_total_cart_weight_max_qty || $ap_fees_ap_total_cart_weight_max_qty == '')) {
                                            $ap_total_cart_weight_rule_valid = 1;
                                        }
                                        //if set condition match true/1 than append rule cost in shipping method's cost
                                        if ($ap_total_cart_weight_rule_valid == 1) {
                                            $is_percent = substr($ap_fees_ap_price_total_cart_weight, -1); // returns "last character"
                                            if ($is_percent == '%') {
                                                $percent = substr($ap_fees_ap_price_total_cart_weight, 0, -1);
                                                $percent = number_format((float)$percent, 2, '.', '');
                                                if (!empty($percent)) {
                                                    //add percent price to shipping cost
                                                    $shipping_rate['cost'] = $shipping_rate['cost'] + ($percent / 100) * $get_cart_total;
                                                }
                                            } else {
                                                //add normal price to shipping cost.
                                                $shipping_rate['cost'] = $shipping_rate['cost'] + $ap_fees_ap_price_total_cart_weight;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                //if advanced pricing for total cart qty exist
//                if (!empty($get_condition_array_ap_class)) {
//
//                    //initialize array for store AP Class valid rules
//                    $class_array_ap = array();
//                    foreach ($get_condition_array_ap_class as $key => $value) {
//                        //store rules in array if class of AP rule selected
//                        if ($value['ap_fees_classes'] != '')
//                            $class_array_ap[$key] = $value; //store all valid rule in array
//                    }
//
//                    //go inside if valid rules found
//                    if (is_array($class_array_ap) && isset($class_array_ap) && !empty($class_array_ap) && !empty($cart_array)) {
//
//                        //get cart data in variable
//                        $woo_cart_array = $cart_array;
//
//                        //get cart total items for percent calculation
//                        $get_cart_total = WC()->cart->get_cart_contents_total();
//
//                        foreach ($class_array_ap as $class) {//loop valid class rule set
//                            //initialize class quantity total
//                            $quantity_class_total = 0;
//                            //get current rule class id/slug
//                            $ap_selected_class_id = $class['ap_fees_classes'];
//                            foreach ($woo_cart_array as $woo_cart_item_key => $woo_cart_item) { //loop cart for count shipping classes
//                                //initialize shipping class
//                                $shipping_class_list = array();
//                                //get current product total classes id
//                                $shipping_classes = get_the_terms($woo_cart_item['product_id'], 'product_shipping_class');
//
//                                if ($shipping_classes) { //if class assigned to product
//                                    foreach ($shipping_classes as $shipping_class) {
//
//                                        $shipping_class_list[] = $shipping_class->slug;
//                                    }
//                                }
//                                //check if class assigned or not
//                                if (!empty($shipping_class_list) && is_array($shipping_class_list)):
//                                    //check if rule class exist in current cart item than increase the cat quantity
//                                    if (in_array($ap_selected_class_id, $shipping_class_list))
//                                        $quantity_class_total += $woo_cart_item['quantity']; //increase class as quantity in item
//                                endif;
//                            }
//                            //store all array key to variable
//                            $ap_fees_ap_class_min_qty = $class['ap_fees_ap_class_min_qty'];
//                            $ap_fees_ap_class_max_qty = $class['ap_fees_ap_class_max_qty'];
//                            $ap_fees_ap_price_class = $class['ap_fees_ap_price_class'];
//                            $ap_fees_classes = $class['ap_fees_classes'];
//
//                            //check if quantity min or max match and cost greater than 0 than it will go for further process
//                            if (($ap_fees_ap_class_min_qty != '' || $ap_fees_ap_class_max_qty != '') && $ap_fees_ap_price_class != 0 && $ap_fees_classes != '') {
//
//                                //initialize rule flag to 0 means false and not apply
//                                $ap_class_rule_valid = 0;
//
//                                if ($quantity_class_total > 0){
//                                    if (
//                                        ($quantity_class_total >= $ap_fees_ap_class_min_qty || $ap_fees_ap_class_min_qty == '') &&
//                                        ($quantity_class_total <= $ap_fees_ap_class_max_qty || $ap_fees_ap_class_max_qty == '')
//                                    ) {
//                                        //if condition match than set flag true for add cost in shipping method
//                                        $ap_class_rule_valid = 1;
//                                    }
//                                    //if set condition match true/1 than append rule cost in shipping method's cost
//                                    if ($ap_class_rule_valid == 1) {
//
//                                        $is_percent = substr($ap_fees_ap_price_class, -1); // returns "last character"
//                                        if ($is_percent == '%') {
//                                            $percent = substr($ap_fees_ap_price_class, 0, -1);
//                                            $percent = number_format((float)$percent, 2, '.', '');
//                                            if (!empty($percent)) {
//                                                //add percent price to shipping cost
//                                                $shipping_rate['cost'] = $shipping_rate['cost'] + ($percent / 100) * $get_cart_total;
//                                            }
//                                        } else {
//                                            //add normal price to shipping cost.
//                                            $shipping_rate['cost'] = $shipping_rate['cost'] + $ap_fees_ap_price_class;
//                                        }
//                                    }
//                                }
//                            }
//                        }
//                    }
//                }

                /* Apply Advanced Pricing rules Cost end here */
                // Add the rate
                if ($has_costs) {
                    //check if calculation of cost go to minus or not
                    /* Start ordering issue and highest, smallest, forceall shipping issue code */
                    if ($get_what_to_do_method == 'force_all') {
                        $currency_symbol = get_woocommerce_currency_symbol();
                        $total_shipping_cost_force_all = $total_shipping_cost_force_all + $shipping_rate['cost'];
                        if ($total_shipping_cost_force_all < 0) {
                            $shipping_price_minus = substr($total_shipping_cost_force_all, 1); //remove - sign for display purpose
                            $shipping_price_minus = number_format((float)$shipping_price_minus, 2, '.', '');
                            //get current currency symbol
                            //customize label of shipping method
                            $shipping_rate['label'] = $shipping_rate['label'] . ': <span class="woocommerce-Price-amount amount"><span class="woocommerce-Price-currencySymbol">' . $currency_symbol . ' -</span>' . $shipping_price_minus . '</span>';
                        }

                        $force_all_shipping_rate[$shipping_rate['id']] = $shipping_rate['label'] . "||" . number_format((float)$shipping_rate['cost'], 2, '.', '') . "||" . $shipping_rate['taxes'];
                        if ($shipping_method_key == 'forceall') {
                            $force_all_cost = 0;
                            $force_all_label = '';
                            $force_all_taxes = 0;

                            /*Default Shipping Package - 23-05-2019*/
                            $new_forceall_label = "";
                            $new_forceall_cost = 0;
                            $new_forceall_tax = 0;
                            $shipping_tax       = 0;
                            /*Default Shipping Package - 23-05-2019*/
                            foreach ($force_all_shipping_rate as $key => $value) {
                                if ($key != 0) {
                                    $value_explode = explode('||', $value);
                                    if (!empty($sitepress)) {
                                        $my_current_lang = (defined('ICL_LANGUAGE_CODE')) ? ICL_LANGUAGE_CODE : $default_lang;
                                        $frc_labl = apply_filters('wpml_translate_single_string', $value_explode[0], 'advanced-flat-rate-shipping-for-woocommerce', $value_explode[0], $my_current_lang);
                                    } else {
                                        $frc_labl = $value_explode[0];
                                    }
                                    
                                    if ( WC()->cart->tax_display_cart == 'incl' ) {
                                        $applied_Rate = WC_Tax::get_rates();

                                        $rate_array = array();
                                        if ( isset( $applied_Rate ) && is_array( $applied_Rate ) ) {
                                            foreach ( $applied_Rate as $rate_arr ) {
                                                $rate_array['rate']     = $rate_arr['rate'];
                                                $rate_array['shipping'] = $rate_arr['shipping'];
                                            }
                                        }
                                        if ( ! empty( $rate_array ) && is_array( $rate_array ) ) {
                                            //add tax value here
                                            $sm_taxable = get_post_meta( $key, 'sm_select_taxable', true );

                                            if ( 'yes' === $rate_array['shipping'] && 'yes' === $sm_taxable) {
                                                $shipping_tax = ( $value_explode[1] * ( floatval( $rate_array['rate'] ) / 100 ) );
                                            }
                                        }
                                    }

                                    if ( isset( $combine_default_shipping_with_forceall ) && $combine_default_shipping_with_forceall == 'yes' ) {
                                        $force_all_label = $force_all_label . '( ' . $frc_labl . ' : ' . $currency_symbol . '' . ( number_format( $value_explode[1] + $shipping_tax, 2 ) ) .
                                                           ' ) ';
                                    } else {

                                        $force_all_label = $force_all_label . '( ' . $frc_labl . ' : ' . $currency_symbol . '' . ( number_format( $value_explode[1] + $shipping_tax, 2 ) ) . ' ) ';
                                    }

                                    $force_all_cost = $force_all_cost + $value_explode[1];
                                    $force_all_taxes = (int)$force_all_taxes + (int)$value_explode[2];

                                    /*Default Shipping Package - 23-05-2019*/
                                    if (isset($combine_default_shipping_with_forceall) && $combine_default_shipping_with_forceall == 'yes') {
                                        $package_rate_label = "";
                                        $package_rate_cost = 0;
                                        $package_rate_tax = 0;
                                        foreach ($package['rates'] as $package_key => $package_val) {
                                            $package_rate_label = $package_rate_label . '\n( ' . $package_val->label . ' : ' . $currency_symbol . '' . $package_val->cost . ' ) ';
                                            $package_rate_cost = $package_rate_cost + $package_val->cost;
                                            $package_rate_tax = (int)$package_rate_tax + (int)$package_val->taxes;
                                        }

                                        $new_forceall_label = $force_all_label . " " . $package_rate_label;
                                        $new_forceall_cost = $force_all_cost + $package_rate_cost;
                                        $new_forceall_tax = $force_all_taxes + $package_rate_tax;
                                    }
                                    /*Default Shipping Package - 23-05-2019*/
                                }
                            }
                            /*Default Shipping Package - 23-05-2019*/
                            if (isset($combine_default_shipping_with_forceall) && $combine_default_shipping_with_forceall == 'yes') {
                                $force_all_shipping_rate_pass_rate = array('id' => 'forceall', 'label' => $new_forceall_label, 'cost' => $new_forceall_cost, 'taxes' => $new_forceall_tax);
                            } else {
                               $force_all_shipping_rate_pass_rate = array('id' => 'forceall', 'label' => $force_all_label, 'cost' => $force_all_cost, 'taxes' => $force_all_taxes);
                            }
                            /*Default Shipping Package - 23-05-2019*/
                            $this->add_rate($force_all_shipping_rate_pass_rate); //apply rate in car
                        }
                        /* END ordering issue and highest, smallest, forceall shipping issue code */
                    } else {
                        if ($shipping_rate['cost'] < 0) {
                            $shipping_price_minus = substr($shipping_rate['cost'], 1); //remove - sign for display purpose

                            $shipping_price_minus = number_format((float)$shipping_price_minus, 2, '.', '');
                            //get current currency symbol
                            $currency_symbol = get_woocommerce_currency_symbol();
                            //customize label of shipping method
                            $shipping_rate['label'] = $shipping_rate['label'] . ': <span class="woocommerce-Price-amount amount"><span class="woocommerce-Price-currencySymbol">' . $currency_symbol . ' -</span>' . $shipping_price_minus . '</span>';
                        }

                        $this->add_rate($shipping_rate); //apply rate in cart
                    }
                }
                do_action('woocommerce_' . $this->id . '_shipping_add_rate', $this, $shipping_rate, $package);
            }
        }
    }

    /**
     * Match methods.
     *
     * Check all created AFRSM shipping methods have a matching condition group.
     *
     * @since 3.0.0
     *
     * @param  array $package List of shipping package data.
     * @return array          List of all matched shipping methods.
     */
    public function afrsm_match_methods($package)
    {
        $matched_methods = array();
        $sm_args = array(
            'post_type' => 'wc_afrsm',
            'posts_per_page' => -1,
            'orderby' => 'menu_order',
            'order' => 'ASC'
        );
        $sm_posts = get_posts($sm_args);

        foreach ($sm_posts as $sm_post) {

            // Check if shipping method conditions match
            $is_match = $this->afrsm_match_conditions($sm_post, $package);

            // Add to matched methods array
            if ($is_match == true) {
                $matched_methods[] = $sm_post->ID;
            }
        }
        return $matched_methods;
    }

    /**
     * Match conditions.
     *
     * Check if conditions match, if all conditions in one condition group
     * matches it will return TRUE and the shipping method will display.
     *
     * @since 1.0.0
     *
     * @param array $sm_post_data
     * @param  array $package List of shipping package data.
     *
     * @return BOOL                    TRUE if all the conditions in one of the condition groups matches true.
     */
    public function afrsm_match_conditions($sm_post_data = array(), $package = array())
    {

        if (empty($sm_post_data)) {
            return false;
        }

        if (!empty($sm_post_data)) {
            $final_condition_flag = apply_filters('afrsm_condition_match_rules', $sm_post_data, $package);
            if ($final_condition_flag) {
                return true;
            }
        }

        return false;
    }

    /**
     * Get items in package.
     * @param  array $package
     * @return int
     */
    public function get_package_item_qty($package)
    {
        $total_quantity = 0;
        foreach ($package['contents'] as $item_id => $values) {
            if ($values['quantity'] > 0 && $values['data']->needs_shipping()) {
                $total_quantity += $values['quantity'];
            }
        }
        return $total_quantity;
    }

    /**
     * Evaluate a cost from a sum/string.
     * @param  string $shipping_cost_sum
     * @param  array $args
     * @return string
     */
    protected function evaluate_cost($shipping_cost_sum, $args = array())
    {
        include_once('class-wc-extra-flat-eval-math.php');

        // Allow 3rd parties to process shipping cost arguments
        $args = apply_filters('woocommerce_evaluate_shipping_cost_args', $args, $shipping_cost_sum, $this);
        $locale = localeconv();
        $decimals = array(wc_get_price_decimal_separator(), $locale['decimal_point'], $locale['mon_decimal_point']);
        $this->fee_cost = $args['cost'];

        // Expand shortcodes
        add_shortcode('fee', array($this, 'fee'));

        $shipping_cost_sum = do_shortcode(str_replace(array('[qty]', '[cost]'), array($args['qty'], $args['cost']), $shipping_cost_sum));

        remove_shortcode('fee', array($this, 'fee'));

        // Remove whitespace from string
        $shipping_cost_sum = preg_replace('/\s+/', '', $shipping_cost_sum);

        // Remove locale from string
        $shipping_cost_sum = str_replace($decimals, '.', $shipping_cost_sum);

        // Trim invalid start/end characters
        $shipping_cost_sum = rtrim(ltrim($shipping_cost_sum, "\t\n\r\0\x0B+*/"), "\t\n\r\0\x0B+-*/");

        // Do the math
        return $shipping_cost_sum ? WC_Eval_Math_Extra::evaluate($shipping_cost_sum) : 0;
    }

    /**
     * Finds and returns shipping classes and the products with said class.
     * @param mixed $package
     * @return array
     */
    public function find_shipping_classes($package)
    {
        $found_shipping_classes = array();

        foreach ($package['contents'] as $item_id => $values) {
            if ($values['data']->needs_shipping()) {
                $found_class = $values['data']->get_shipping_class();

                if (!isset($found_shipping_classes[$found_class])) {
                    $found_shipping_classes[$found_class] = array();
                }

                $found_shipping_classes[$found_class][$item_id] = $values;
            }
        }
        return $found_shipping_classes;
    }

    public function fee_array_column(array $input, $columnKey, $indexKey = null)
    {
        $array = array();
        foreach ($input as $value) {
            if (!isset($value[$columnKey])) {
                trigger_error("Key \"$columnKey\" does not exist in array");
                return false;
            }
            if (is_null($indexKey)) {
                $array[] = $value[$columnKey];
            } else {
                if (!isset($value[$indexKey])) {
                    trigger_error("Key \"$indexKey\" does not exist in array");
                    return false;
                }
                if (!is_scalar($value[$indexKey])) {
                    trigger_error("Key \"$indexKey\" does not contain scalar value");
                    return false;
                }
                $array[$value[$indexKey]] = $value[$columnKey];
            }
        }
        return $array;
    }

    /**
     * Work out fee (shortcode).
     * @param  array $atts
     * @return string
     */
    public function fee($atts)
    {
        $atts = shortcode_atts(array('percent' => '', 'min_fee' => ''), $atts);

        $atts['percent'] = $this->string_sanitize($atts['percent']);
        $atts['min_fee'] = $this->string_sanitize($atts['min_fee']);

        $calculated_fee = 0;

        if ($atts['percent']) {
            $calculated_fee = $this->fee_cost * (floatval($atts['percent']) / 100);
        }

        if ($atts['min_fee'] && $calculated_fee < $atts['min_fee']) {
            $calculated_fee = $atts['min_fee'];
        }

        return $calculated_fee;
    }

    public function string_sanitize($string)
    {
        $result = preg_replace("/[^ A-Za-z0-9_=.*()+\-\[\]\/]+/", "", html_entity_decode($string, ENT_QUOTES));
        return $result;
    }

}
